#!/usr/bin/env ipython

'''
This file has date & time utilities, especially in the context of quotes.
It can be used standalone or in conjunction with other modules for more
conveninet operations.

NOTE. any datetime object here (dtobj or date_obj) is of type dt.datetime.
That has a bunch of other properties if needed. Eg, to convert a dtobj
curr_datetime to a different timezone such as UTC:
    >> curr_datetime.astimezone(pytz.timezone('UTC'))
'''


import re
import datetime as dt
from datetime import date, timedelta
from pytz import timezone, all_timezones

#import time as t
#import dateutil as du
#import dateutil.parser as dp

import logging
logger = logging.getLogger()


_DAY_ = {6: 'sunday', 0: 'monday', 1: 'tuesday', 2: 'wednesday', 3: 'thursday',
            4: 'friday', 5: 'saturday'}

_MTH_ = {'jan': '01', 'feb': '02', 'mar': '03', 'apr': '04', 'may': '05', 'jun': '06',
         'jul': '07', 'aug': '08', 'sep': '09', 'oct': '10', 'nov': '11', 'dec': '12' }


_DAY_BRIEF_ = {6: 'sun', 0: 'mon', 1: 'tue', 2: 'wed', 3: 'thu', 4: 'fri', 5: 'sat'}


#################################################################################
# Utility and help functions that could be imported by other modules
#################################################################################

def curr_time(loc=None):
    """Returns the current time in a specifc format:  day hr:min:sec, eg:

       If loc is not specified, localtime() is used, else one of various
       options for loc: [utc|eastern|pacific|central] entered as a string
       OR any valid timezone in pytz.all_timezones will do , eg: 'US/Mountain'

       print curr_time() -> 'Sun 16:26:20'
       print curr_time('US/Eastern) -> 'Sun 19:26:20'
       print curr_time('utc') -> 'Mon 00:26:20'         """

    # other ways to get time
    #   curr = t.localtime(t.time())
    #   t.asctime(t.gmtime()) for human version
    if loc == None: curr = dt.datetime.now()
    elif loc in all_timezones: curr = dt.datetime.now(loc)
    elif loc.lower() == 'utc':  curr = dt.datetime.now(timezone('UTC'))
    elif loc.lower() == 'eastern': curr = dt.datetime.now(timezone('US/Eastern'))
    elif loc.lower() == 'central': curr = dt.datetime.now(timezone('US/Central'))
    elif loc.lower() == 'pacific': curr = dt.datetime.now(timezone('US/Pacific'))
    else:
        logger.error('Unsupported location: {}'.format(loc))
        return 'Sun 00:00:00'

    return curr.strftime('%a %H:%M:%S')



def curr_date(loc=None):
    """Returns the current date. This is similar to curr_time and is timezone aware
    via loc. See the curr_time() function for description of loc.

    Returns a tuple of following form (2014, 11, 9, 'Sunday'). Note the lack of
    zero-padding"""

    if loc == None: curr = dt.datetime.now()
    elif loc in all_timezones: curr = dt.datetime.now(loc)
    elif loc.lower() == 'utc':  curr = dt.datetime.now(timezone('UTC'))
    elif loc.lower() == 'eastern': curr = dt.datetime.now(timezone('US/Eastern'))
    elif loc.lower() == 'central': curr = dt.datetime.now(timezone('US/Central'))
    elif loc.lower() == 'pacific': curr = dt.datetime.now(timezone('US/Pacific'))
    else:
        logger.error('Unsupported location: {}'.format(loc))
        return (0,0,0,'Sunday')

    return curr.year, curr.month, curr.day, curr.strftime('%A')



def curr_dt(loc=None):
    """Returns the current date&time as a datetime object. The calling API is similar to
    curr_time and is timezone aware via loc. See the curr_time() function for description
    of loc.

    HOWEVER, the return type is a dt.datetime tuple format w/timezone spec"""

    if loc == None: curr = dt.datetime.now()
    elif loc in all_timezones: curr = dt.datetime.now(loc)
    elif loc.lower() == 'utc':  curr = dt.datetime.now(timezone('UTC'))
    elif loc.lower() == 'eastern': curr = dt.datetime.now(timezone('US/Eastern'))
    elif loc.lower() == 'central': curr = dt.datetime.now(timezone('US/Central'))
    elif loc.lower() == 'pacific': curr = dt.datetime.now(timezone('US/Pacific'))
    else:
        logger.error('Unsupported location: {}'.format(loc))
        return dt.dateime(1900,1,1,tzinfo=timezone('UTC'))

    return curr



def first_of_month():
    """Gets the first of a month. This is NOT timezone aware""" #TODO.
    d = date.today()
    return date(d.year, d.month, 1)


def get_first_day(dtobj, d_years=0, d_months=0):
    """d_years, d_months are "deltas" to apply to dtobj"""
    y, m = dtobj.year + d_years, dtobj.month + d_months
    a, m = divmod(m-1, 12)
    return date(y+a, m+1, 1)


def get_last_day(dtobj):
    """Last day of month, given a date object"""
    return get_first_day(dtobj, 0, 1) + timedelta(-1)


def is_weekday(dtobj):
    """True if dtobj is not a Saturday or Sunday"""
    return dtobj.weekday() in range(0,4)


def is_weekend(dtobj):
    """True if dtobj is a Saturday or Sunday"""
    return dtobj.weekday() in (5,6)


def is_saturday(dtobj):
    """True if dtobj is a Saturday"""
    return dtobj.weekday() == 5


def is_sunday(dtobj):
    """True if dtobj is a Sunday"""
    return dtobj.weekday() == 6



def format_date( date_obj ):  # date_obj is a datetime.date object
    """Returns a formatted date string given a date object. If the day is
    the first of the month, then the day is dropped, eg:
        format_date ( datetime.date(2013, 04, 12)) returns 2013-04-12
        format_date ( datetime.date(2013,  4,  1)) returns 2013-04
    """
    if date_obj.day == 1:
        full_date = date_obj.isoformat()
        return full_date[0:7]
    else:
        return date_obj.isoformat()




def parse_date( adate ):
    """Returns a date object constructed by datetime(year, month, day) where
    year, month & day are parsed from informal input.
    Acceptable input will have one of two sequences(separated by '-' for '.')
        YYYY-MM-DD if ALL are numeric, or MM-DD-YY if alpha numeric eg:
        20140918    # must be 8 characters
        2014-09-18
        2014.9      # leading 0 is optional for month & day
        jul9.2014
        jul09.14    # leading prefix 20 is optional for year
        july-2014
        july-14     # This is confusing as hell though!! DO NOT USE
        9.18.14     # This is ambiguous...
        9/18/14     # This is ok
        09/18/2014  # This is ok
    """
    d = date.today()
    dflt_date = date(d.year, d.month, 1)   # pick 1st of month by default

    #<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    def get_md(monthday):   # return month, day tuple from a string
        if monthday.isalpha():  # we have month only
            day = 1
            mth_str = monthday[0:3].lower()
        else:           # both month and day are in here
            day = re.findall(r'\d+', monthday) [0]
            mth_str = re.split(r'\d+', monthday) [0]
            mth_str = mth_str[0:3].lower()

        try: month = _MTH_[mth_str]
        except KeyError, e:
            print "ERROR. Cannot find equivalent numeric for month", e
            month = date.today().month

        return int(month), int(day)

    def pad_year(yr):
        if   len(yr) == 1: return 2000 + int(yr) #    4 => 2004
        elif len(yr) == 2: return 2000 + int(yr) #   13 => 2013 scope for confusion 1913 or 2013?
        elif len(yr) == 3: return 2000 + int(yr) #  137 => 0    this is an error
        else:              return int(yr)        # 1995         cleanest
    #<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    x = str(adate).lower().strip()

    if x.isdigit(): #* must be all numeric, eg: 20130412
        if len(x) == 8:
            fyear, fmonth, fday = int(x[0:4]), int(x[4:6]), int(x[6:8])
        else:
            print "ERROR. unmatched date format; needs to be 8 chars if specified this way", x
            return dflt_date

    else:           #* needs parsing

        if '/' in x: # slash '/' based is different from '-' or '.' based
            slash_based = True
            if '.' not in x and '-' not in x:
                y = re.split('/', x)
            else:
                print "ERROR. Cannot mix and match '/' with '.' , or '/' with '-'. "
                return dflt_date
        else:
            slash_based = False
            y = re.split('[-.]', x)

        digits = [0 if i.isdigit() else -1 for i in y]
        all_digits = True if sum(digits) == 0 else False


        if all_digits:  #** sequence must be year, month, day
            if len(y) == 3:
                if slash_based:
                    fyear, fmonth, fday = int(pad_year(y[2])), int(y[0]), int(y[1]) # eg, 5/18/13
                else:
                    fyear, fmonth, fday = int(y[0]), int(y[1]), int(y[2]) # eg, 2013-05-18
            elif len(y) == 2:
                if slash_based:
                    fyear, fmonth, fday = int(pad_year(y[1])), int(y[0]), 1 # eg, 5/13
                else:
                    fyear, fmonth, fday = int(y[0]), int(y[1]), 1 # eg, 2013-05

            elif len(y) < 2:
                print "ERROR. Not enough information to construct date. Passing default"
                return dflt_date
            else:
                print "WARNING. Ignoring info beyond given year, month, day"

        else:           #** sequence must be month, day, year
            if slash_based:
                print 'ERROR. A "/" based date must consist ONLY of digits in month/day/year format'
                return dflt_date
            if len(y) == 3:  # eg, jan-14.2013  or may.25.15, etc
                fyear = pad_year(y[2])
                fmonth, fday = get_md( y[0]+y[1] )
            elif len(y) == 2:  # eg, jan.14 or jan3-14
                fyear = pad_year(y[1])
                fmonth, fday = get_md( y[0] )
            elif len(y) == 1:
                print "ERROR. Not enough information to construct date. Passing default"
                return dflt_date
            else:
                print "WARNING. Ignoring info beyond given month, day, year"

    # finally we got this far...
    try: date_obj = date(fyear, fmonth, fday)
    except Exception, e:
        print "ERROR. ", e
        return dflt_date

    return date_obj



def markets_open():
    """Return True if markets are open. Uses the NYC/Eastern time to figure this out"""

    wallst_time = dt.datetime.now(timezone('US/Eastern'))
    chour  =  wallst_time.hour
    cminute =  wallst_time.minute
    cweekday =  wallst_time.weekday()
    chour_frac  =  chour + cminute/60.0

    closed = (cweekday in (5,6)) or (chour_frac < 6.5) or (chour > 13)
    return False if closed else True



def markets_closed():
    """Return True if markets are closed"""
    return not markets_open()



def exact_exp_date(results, exp):
    """Returns a string of unique expiry date in yyyy-mm-dd form
    results: tuple of (Ticker[0] Strike[1] Type[2] Expiration[3] Last[4] Bid[5] Ask[6])
        exp: expiration date that was entered or inferred. however, may not have
             day of the month, or may resolve to multiple dates."""
    exp_dates = set([])
    for result in results[1:]:
        exp_dates.add(result[3])

    if len(exp_dates) == 0:
        print "WARNING. No matching date found! Try again."

    if len(exp_dates) != 1:
        print "WARNING. multiple dates found! "
        for d in exp_dates: print "  ->   ", d
        print "Try again with exact one."
    exp_date = ' '.join(list(exp_dates))
    return exp_date


def iso_date(adate):
    """Returns True if adate is in form of 'yyyy-dd-mm' else False"""
    if type(adate) != str or len(str(adate)) !=10 : return False
    alist = adate.split('-')
    if len(alist) != 3: return False

    y,m,d = alist
    if y.isdigit() and len(y) == 4 and \
       m.isdigit() and len(m) == 2 and \
       d.isdigit() and len(d) == 2: return True
    else: return False


def iso_date2(adate):
    """Returns int tuple (yyyy,mm,dd) if adate is in form of 'yyyy-dd-mm' else (0,0,0)"""
    if type(adate) != str or len(str(adate)) !=10 : return (0,0,0)
    alist = adate.split('-')
    if len(alist) != 3: return  (0,0,0)

    y,m,d = alist
    if y.isdigit() and len(y) == 4 and \
       m.isdigit() and len(m) == 2 and \
       d.isdigit() and len(d) == 2: return (int(y), int(m), int(d))
    else: return (0,0,0)




#################################################################################
# Old functions kept for reference - TEMPORARILY
#################################################################################
# def curr_date_old():    #TODO. make this obsolete
#     curr   = t.localtime(t.time())
#     wday   = _DAY_[curr.tm_wday] # week day
#     day    = curr.tm_mday         # day of month
#     year   = int(curr.tm_year)
#     month  = curr.tm_mon
#     print "types", type(year), type(month), type(day), type(wday)  # 2013, 04, 12, friday
#     return year, month, day, wday  # 2013, 04, 12, friday
#
#
# def markets_open_old():
#     """Return True if markets are open. Uses naive time"""
#
#     curr = t.localtime(t.time())
#     curr_hour = float(curr.tm_hour) + float(curr.tm_min)/60.0
#     time_str = str(curr.tm_hour) + ':' + str(curr.tm_min) + ':' + str(curr.tm_sec)
#     ## # print 'current weekday and time', _DAY_[curr.tm_wday], time_str    # DEBUG
#
#     # After 1 pm or before 6:30 am, markets are closed
#     closed = curr_hour > 13.0 or curr_hour < 6.5 or \
#         _DAY_[curr.tm_wday] == 'Saturday' or _DAY_[curr.tm_wday] == 'Sunday'
#
#     return False if closed else True
#
#
# def new_parse_date( adate ):    # TODO. Is this better than the existing one?
#     # return du.parser.parse(adate)
#     # print '-- adate = ', adate, '--'  // has bugs, eg, may23.12 becomes 05/23/2014
#     return dp.parse(adate).date()




# *********************************************************
# Main Routine (& test functions)
# *********************************************************
def test_timeutils(args):
    print 'Curr time:' , curr_time()
    print 'Curr date:' , curr_date()
    print format_date( date(2013, 04, 12) )
    print format_date( date(2013, 04, 1) )
    print "Market open  ? ", markets_open()
    print "Market closed? ", markets_closed()
    print 'GOOD DATES\n', "***********"
    print 1, parse_date('jul23.14')
    print 2, parse_date('20140918')
    print 3, parse_date('2014-09-18')
    print 4, parse_date('2014.9')
    print 5, parse_date('jul9.2014')
    print 6, parse_date('jul09.14')
    print 7, parse_date('july-2014')
    print 8, parse_date('july-14')
    print 'BAD DATES\n', "***********"
    print 1, parse_date('2014091')
    print 2, parse_date('113-13-59')
    print 3, parse_date('01131359')
    print 4, parse_date('jux.14')


if __name__ == '__main__':
    import sys
    test_timeutils(sys.argv[1:])



#!/usr/bin/env python2.7

'''Tests utils module for ease of running & integrating test results.
   Execute this function instead of the standard ut.main(), eg:
        run_tests(TestTimeutils)
'''

import unittest as ut

# ------------------------------------------------------------ #
def twrap(func):
    """Decorator for test functions to print out what test is being run"""
    def inner(*args):
        print "\n-- RUNNING TEST: <{}> --\n".format(func.__name__)
        return func(*args)
    return inner


# ------------------------------------------------------------ #
def run_tests(test_case):
    loader = ut.TestLoader().loadTestsFromTestCase(test_case)
    result = ut.TestResult()
    loader(result)
    print "=" * 60
    print "TEST RESULTS FOR", test_case.__name__

    print "*FAILURES*"
    print "-" * 60

    for t, f in result.failures:
        print "->", t, "<-"
        print f
    for t, e in result.errors:
        print "->", t, "<-"
        print e

    print "*ERRORS*"
    print "-" * 60

    tcount = result.testsRun
    fcount = len(result.failures)
    ecount = len(result.errors)
    pcount = tcount - fcount - ecount
    prate = round( 100.0* pcount/float(tcount), 2)

    summary = {}
    summary['Name'       ] = test_case.__name__
    summary['Pass'       ] = pcount
    summary['Failures'   ] = fcount
    summary['Errors'     ] = ecount
    summary['Total'      ] = tcount
    summary['Pass rate'  ] = prate

    # if result.wasSuccessful():    # relegate this to print_summary()
    print_summary(summary)

    return summary, result


# ------------------------------------------------------------ #
def append_summary(master, summary):
    if not master:
        master = summary.copy()
        master['Name'] = 'ALL TESTS'
        master['Sublist'] = [summary.copy()]     # store list of all test summaries
        return master

    for k in master.keys():
        if k == 'Sublist':
            master['Sublist'].append(summary.copy())
        else:
            master[k] += summary[k]

    if not master.has_key('Sublist'):
        master['Sublist'] = [summary.copy()]     # store list of all test summaries

    master['Pass rate'] = round( 100.0* master['Pass']/float(master['Total']), 2)
    master['Name'] = 'ALL TESTS'

    return master


# ------------------------------------------------------------ #
def print_breakdown(slist):
    """Prints breakdown of other tests in slist in shortened form"""
    print '   Name', ' '*20, 'Pass/Total  Fail Error PassRate '
    for s in slist: # slist is list of dict()s
        n = s['Name']
        p = s['Pass']
        f = s['Failures']
        e = s['Errors']
        t = s['Total']
        pr = str(s['Pass rate']) + '%'

        spc = ' '*(24 - len(n))
        print '   %s %s    %03d/%03d   %03d   %03d   %s ' % (n, spc, p, t, f, e, pr)


# ------------------------------------------------------------ #
def print_summary(summary, title=None):
    print
    if summary == {}:
        print "   WARNING. No tests available for {}".format(title)
        print "-" * 60
        return

    if title==None:
        title = "SUMMARY FOR '%s'" % (summary['Name'])

    print "*%s*" % (title)
    print "-" * 60


    if summary.has_key('Sublist'):
        print_breakdown(summary['Sublist'])

    print "-" * 60
    print "    Name       :", summary['Name']
    print "    Pass       :", summary['Pass']
    print "    Failures   :", summary['Failures']
    print "    Errors     :", summary['Errors']
    print "    Total count:", summary['Total']
    print "    Pass rate  :", str(summary['Pass rate']) + '%'
    print "-" * 60

    print
    s = summary
    if summary['Pass'] == summary['Total']:
        print "All tests ran without problems :)"
    else:
        print s['Pass'], "out of", s['Total'], "tests ran without problems. There were", s['Errors']+s['Failures'], "fails or errors"
    print
    print "=" * 60



#!/usr/bin/env python2.7
# Run as:
# python setup.py install --user    [--user needed if no root permissions]

#from distutils.core import setup   - DO NOT USE; DUMPS EVERTYHING UNDER SITE-PACKAGES & UNCLEAN INSTALL

from setuptools import setup, find_packages
import mhut

setup(
    name = 'mhut',   # package name
    author = 'Mahmud Hassan',
    version = mhut.__version__,
    description='Various utilities by MH that show up often in his use',
    scripts = ['scripts/pathutils'],
    packages = find_packages(exclude=('testdir',)),
    py_modules = ['testutils', 'timeutils', 'datautils', 'msgutils', 'pathutils']
)



#!/usr/bin/env python
"""Author : Mahmud Hassan
   Purpose: Automates common shell functions for env variables
   Usage  : path_utils.py <cmd> [args]
            where <cmd> is one of: 'add', 'remove', 'clean', or 'list'
            and [args] specify the paths, or optionally the
            environment variable as follows: 

               path_utils.py add    [env var] <path 1> <path 2> ... <path N> 
               path_utils.py remove [env var] <path 1> <path 2> ... <path N> 
               path_utils.py clean  [env var] 
               path_utils.py list   [env var] 

            if 'env var' is not specified the commands apply to
            PATH by default. 'env var' may be upper case or lower case.

"""

import sys
import os





# **********************************************************
# Begin class definition for Env
# **********************************************************

class Env:
    """Object for environment variable. Can perform operations such as
    add, clean, remove or list paths for that object"""

    def __init__(self, name):
        """Creates & initiliazes env var object along with path list"""
        self.name = name
        self.val = os.getenv(name)
        if self.val != None:
            self.path_list = self.val.split(':')
        else:
            os.environ[name] = name  # create a new env variable 
            self.path_list = []


    def padd(self, path, pos=0):
        """Inserts path into current variable - into beginning(0) by default
        but could be at end (-1) or any other position"""
        path = os.path.normpath(path)           # remove double slashes and stuff
        if path in self.path_list:
            print path, "already exists. Not appending to ", self.name
        elif os.path.exists(path):
            self.path_list.insert(pos, path)
        else:
            print path, "does not exist! Not appending to ", self.name
            return
        self.pupdate()


    def plist(self):
        """Just lists out paths in env variable, one per line"""
        if self.val == None:
            print "No such env variable ", self.val, " exists!"
        else:
            print "Listing for ", self.name
            for p in self.path_list: print "  ", p


    def pclean(self):
        """Cleans up path to remove dead directories, duplicates, etc."""
        path_list_pruned = []
        for p in self.path_list:
            if not os.path.exists(p):
                print "Does not exist! ", p
            elif p in path_list_pruned:
                print "Duplicate found ", p
            else:
                p = os.path.normpath(p)       # remove double slashes and stuff
                path_list_pruned.append(p)

        self.path_list = path_list_pruned
        self.pupdate()


    def premove(self, path):
        """Removes all occurences of path from env variable"""
        path = os.path.normpath(path)       # remove double slashes and stuff
        if path not in self.path_list:
            print "Not found in path list! ", path
        else:
            print "Removing ", path, " from env var ", self.name 
            while path in self.path_list:    # needed just in case path is not cleaned first
                self.path_list.remove(path)
            self.pupdate()


    def pupdate(self):
        """Updates the internal env val to ensure path_list & val are insync"""
        try:
            tmp = self.path_list[0]
        except IndexError:
            print "Empty value for env variable ", self.name
            return

        for p in self.path_list[1:]:
            tmp = tmp + ':' + p
        self.val = tmp


    def pwrite(self):
        """Writes out sanitized or new path list to file for sourcing"""
        shell = os.getenv('SHELL') 
        if shell == None:   # assume bash or ksh
            shell = 'bash'
        else:
            shell = os.path.basename(shell)

        fname = '/tmp/source_' +  os.environ['USER']   # get login id of current user
        try:
            fid = open(fname, 'w')
        except:
            print "ERROR. Could not open ", fname, " for writing! Exiting..."
            exit(1)

        if self.val == None:
            self.val = ""

        if 'csh' in shell:
            wstr = "setenv " + self.name + " " + self.val
        else:
            wstr = "export " + self.name + "=" + self.val

        fid.write(wstr)
        fid.close()
        print "Source ", fname, " for new path to take effect"

# **********************************************************
# End of class definition for Env
# **********************************************************



def usage(s=None):
    if s != None:
        print s
    print __doc__
    exit()



#TODO: fix this so that errors in one or paths in list of paths is 
# handled more appropriately, eg, if only one non-existent path to
# be added should not attempt to write to tmp file
def execute(cmd, var, args):
    """Executes one of only 4 real commands: add, clean, remove or list
    on an environment variable. Assume arguments have been sanitized"""
    env_obj = Env(var)

    if 'list' in cmd:
        env_obj.plist()
    elif 'clean' in cmd:
        env_obj.pclean()
        env_obj.pwrite()
    elif 'add' in cmd:
        for path in args: env_obj.padd(path)
        env_obj.pwrite()
    elif 'remove' in cmd:
        for path in args: env_obj.premove(path)
        env_obj.pwrite()
    else:
        usage("Some really weird error occured. Check program inputs & try again!")



def process_options(args):
    """Process options based on legal operations & subcommands
    Return sanitized cmds and arguments"""
    subcmds = dict()   # each key(cmd) can take on a val of 0, or 1 
    subcmds_wo_arg = [ 'clean', 'list' ]
    subcmds_with_args = [ 'add', 'remove' ]

    for cmd in subcmds_wo_arg:
        subcmds[cmd] = 0
    for cmd in subcmds_with_args:
        subcmds[cmd] = 1

    if (len(args) == 0):
        usage("ERROR. must have one sub-command available")

    cmd = args.pop(0)
    argc = len(args)

    def bad_args(cmd, argc):
        return True if argc < subcmds[cmd] else False

    env_var = '' 
    # determine what kind of cmd was given and arguments
    if cmd not in subcmds:
        usage("ERROR. Unrecognized cmd " + cmd + "! cmd must be from appropriate list")
    elif bad_args(cmd, argc):
        usage("Must enter at least one argument for " + cmd)
    elif argc > subcmds[cmd]:          # determine if it defaults to PATH or anything else
       if os.getenv(args[0]) != None: 
            env_var = args.pop(0)      
       elif os.getenv(args[0].upper()) != None: 
            env_var = args.pop(0).upper() 
       else: # first argument is NOT a known env variable 
          if (cmd == 'remove'): 
             env_var = 'PATH' 
          elif (cmd == 'add') and  ('/' not in args[0]) and (len(args) > 1):  # not like a path & has at least one other argument
               env_var = args.pop(0)      # assume new env variable to be created 
          else:
               usage("Unrecognized environment variable " + args[0])
    else:
        env_var = 'PATH'

    return (cmd, env_var, args)


def test():
    """Tests out operation of utils with sample commands and strings"""
    # usage()
    path_obj = Env('PATH')
    path_obj.pclean()
    path_obj.padd('/home/mahmud/downloads///')
    path_obj.padd('/home/mahmud/apps//', -1)
    path_obj.premove('/abcd')
    path_obj.premove('/cad/tools/platform/lsf/7.0/linux2.6-glibc2.3-x86_64/etc')
    path_obj.premove('/cad/tools/platform/lsf/7.0/linux2.6-glibc2.3-x86_64/bin')
    path_obj.plist()
    cmd = 'add /usr/bin/'
    cmd = 'clean abcd'
    cmd = 'ld_clean'
    cmd = 'lic_add /bin /tmp'
    cmd = ''
    cmd = 'env_remove CADENCE_PATH /some/arbitrary/dir'
    cmd = 'env_list CADENCE_PATH'
    cmd = 'ld_remove /cad/tools/cliosoft/sos_5.31_linux/lib    /cad/tools/cadence/soc/SOC71/tools/lib'
    (cmd, var, args) = process_options(cmd.split())
    print "Executing: ", cmd, var, args
    execute (cmd, var, args)


def main():
    """Handle argument options, create env object, and perform operations"""
    sys.argv.pop(0)
    (cmd, var, args) = process_options(sys.argv[:])
    execute(cmd, var, args)


if __name__ == '__main__':
    if (len(sys.argv) > 1) and (sys.argv[1] == "test"):  
       test()
    else:
       main()





#!/usr/bin/env python2.7
'''
Data structure utils module for generic use. Has functions
that would be really nice to add to string, dictionary or 
DataFrame types. Could extend some of these to add capability.

Some of the functions need some heavy testing and debug. 
'''

import re
import pandas as pd
import numpy as np 
import logging 
import datetime as dt


_ANYOP_ = re.compile(r'[><=&|]')
logger = logging.getLogger(__name__)


## =================================================================== ##
## ROUNDING FUNCTIONS
## =================================================================== ##
# Simple round off to 2 & 3 dec places wrappers

R2 = lambda x: round(x, 2)

R3 = lambda x: round(x, 3)

## ------------------------------------------------------------------- ##
def RN(num, places=2):
    """Simple round off to 2 dec places by default wrapper. Fails silently 
    on non floats"""
    try: f = round(num, places)
    except: f = num
    return f 

## ------------------------------------------------------------------- ##
def roundoff_list(alist, places=2):
    """Rounds off contents in a list to '2' places by default.  Fails 
    silently on non floats.  NOTE. list is modified INPLACE"""
    for i, v  in enumerate(alist): 
        alist[i] = RN(v, places) 
    return  

## ------------------------------------------------------------------- ##
def roundoff_dict(adict, places=2):
    """Rounds off contents in a dictionary table to '2' places by default.
    Can handle one level deep of list of nums. Fails silently on non floats.
    NOTE. Dictionary is modified INPLACE"""
    for k,v  in adict.items(): 
        if type(v) == list: roundoff_list(v, places)
        else: adict[k] = RN(v, places)
    return  


## ------------------------------------------------------------------- ##
def roundoff_df(df, places=0, columns=None, indices=None):
    """Round off all entries in DataFrame. If no specific columns or
    indices are provided all DataFrame elements are rounded. 
    Returns a DataFrame with rounding applied

    places : number of decimal places to round
    columns: None or list of columns to apply rounding
    indices: None or list of indices to apply rounding
    """

    tmp = df.copy()

    if columns==None and indices==None: # round all
        for j in tmp.columns: 
            tmp[j] = tmp[j].round(places)
    elif columns!=None and indices==None: # round specific columns 
        for j in columns: 
            tmp[j] = tmp[j].round(places)
    elif columns==None and indices!=None: # round specific rows 
        for i in indices: 
            tmp.ix[i] = tmp.ix[i].round(places) 
    else:                                  # specific rows & columns (slow at the moment)
        for i in indices: 
            tmp.ix[i, columns] = tmp.ix[i, columns].round(places) 

    return tmp 


## =================================================================== ##
## DATAFRAME BUILDING FUNCTIONS
## =================================================================== ##
def parse2df(fname):
    '''Returns & generates a DataFrame from a file that has free from
    spacing BUT has the header position specified with "|". Currently not
    changeable but may be modified in the future. Helps deal with non-CSV
    format but that has maligned tabs and spaces. '''

    # NOTE. This is currently done in two passes to make use of exisiting 
    # function txt2df & is a little slow, but ok for small files to parse

    try:
        fin = open (fname)
    except:
        print "Could not open {} for reading!".format(fname)

    txtlist = fin.readlines()    # slurp whole thing
    fin.close()
  
    sep_pragma = '#<pr:sep>'
    txtlist = [t.replace(sep_pragma,' '*len(sep_pragma)) if t.lstrip().startswith(sep_pragma) else t 
                for t in txtlist]
    df = txt2df(''.join(txtlist), header_sep='|', header=True, skip_comment=True, index='default')
    return df


## ------------------------------------------------------------------- ##
def txt2df(strtxt, header_sep='|', header=True, skip_comment=True, index='default', 
           strip_pct=True):
    '''Returns & generates a DataFrame from a string that has free form
    spacing BUT has the header position specified with "|". Currently not
    changeable but may be modified in the future. Helps deal with non-CSV
    format but that has maligned tabs and spaces. 
        strtxt:       blob of text that needs to be parsed 
        header:       [True|False] - has a header or not 
        header_sep:   character marking positions of string separation  
        skip_comment: [True|False] - determine if comments are to be skipped
                      over. Empty lines are always ignored. 
                      currently only '#' is treated as comment 
        index:        use 'default', None, or any other valid column 
        strip_pct:    If True, removes '%' characters from txt.

        Returns:      A pd.DataFrame object
    '''

    positions = []
    lol = []

    sep_found = False 

    for line in strtxt.split('\n'):
        if len(line.strip()) == 0:  # skip blank
            continue
        elif not sep_found and line.lstrip().startswith(header_sep): 
            positions = [i for (i, c) in enumerate(line) if c == header_sep]
            continue
        elif (line.lstrip()).startswith('#'):   # skip comment 
            continue

        line = line.replace('\t', ' ')

        if len(positions) == 0: # use split by space by default, basically csv type
            lol.append( line.strip().split() )
        else:
            startpos = [0] + positions[:]
            endpos = positions[:] + [len(line)]
            tmp = [line[a:b].strip() for a,b in zip(startpos,endpos)]
            if strip_pct:
                tmp[1:] = [t.replace('%','') for t in tmp[1:]]      # Remove % symbols from data text
            lol.append(tmp)

    index_name = lol[0][0] if index == 'default' else index

    df = pd.DataFrame(lol[1:], columns =lol[0])
    if index_name != None:
        df.set_index(index_name, inplace=True)
    return df


## =================================================================== ##
## PRINTING FUNCTIONS
## =================================================================== ##
def pprint_dict(adict, orderlist=[]):
    """Prints out a dictionary (eg stock quote) nicely in desired print orderlist."""

    if adict == {}: return
    keyset = set(adict.keys())
    orderset = set(orderlist)
    if not orderset.issubset(keyset): 
        logger.warn( "Specified order is mismatched. Bad fields will be ignored")
        for k in orderset: 
            if k not in keyset: orderlist.remove(k)
    
    # now print in clean order
    remainset = keyset.symmetric_difference(orderlist)
    maxlen = max( [len(k) for k in keyset] ) 

    printsets = orderlist + list(remainset)
    for k in printsets: 
        v = adict[k]
        if type(v) == pd.Series or type(v) == pd.DataFrame: 
            print '-' * 80
            print v
            print '-' * 80
            continue
        if type(v) == dt.datetime: v = dt.datetime.ctime(v)
        print k.ljust(maxlen), ':', v 


## =================================================================== ##
## OTHER UTILITIES
## =================================================================== ##
def isnumeric(a):
    """Returns True if 'a' is an int, float, or a string that could be converted
    to an int or float"""

    if type(a) == int or type(a) == float: return True
    elif type(a) == str:
        if a.replace('.','').isdigit() and a.count('.') < 2: 
            return True
    elif np.dtype(a) == int or np.dtype(a) == float: return True
    return False


## ------------------------------------------------------------------- ##
def reorder_list(origlist, orderlist, qualifier='any'):
    """
    origlist: Given list of labels eg: ['Strike', 'Exp', 'Ask', 'Bid'] 
    orderlist: List of labels in desired order, eg: ['Exp', 'Strike'] 
    qualifier: One of ['any'|'before'|'after'|'begin'|'end']
        any - any position as longer as order is satisfied
        before - insert labels before position of the last entry
        after - insert labels after position of first entry
        begin - set the order to beginning of list of all labels
        end - set the order to end of the list of all labels
    NOTE1. origlist is expected to have UNIQUE entries ONLY 
    NOTE2. All positions of given orderlist will be adjacent to each other."""

    
    if len(origlist) < 2: return origlist 
    if len(orderlist) < 2: return  origlist
    keylist = origlist[:]       # need copy to avoid messing up input 
    deslist = orderlist[:]      # need copy to avoid messing up input 

    keyset = set(keylist)
    orderset = set(deslist)
    if not orderset.issubset(keyset): 
        logger.warn( "Specified order is mismatched. Bad fields will be ignored")
        for k in orderset: 
            if k not in keyset: deslist.remove(k)
    
    if qualifier in ('any'): 
        idx = keylist.index(deslist[0])
        for k in deslist: keylist.remove(k)
        keylist = keylist[:idx] + deslist + keylist[idx:]

    if qualifier in ('after'): 
        idx = keylist.index(deslist[0])
        firstleg, lastleg = keylist[:idx], keylist[idx+1:] 
        for k in deslist[1:]: lastleg.remove(k)
        keylist = firstleg + deslist + lastleg 

    if qualifier == 'before':
        idx = keylist.index(deslist[-1])
        firstleg, lastleg = keylist[:idx], keylist[idx+1:] 
        for k in deslist[:-1]: firstleg.remove(k)
        keylist = firstleg + deslist + lastleg 

    if qualifier == 'begin':
        for k in deslist: keylist.remove(k)
        keylist = deslist + keylist 

    if qualifier == 'end':
        for k in deslist: keylist.remove(k)
        keylist = keylist + deslist 
   
    return keylist



## ------------------------------------------------------------------- ##
def df_reorder_columns(df, orderlist=[], qualifier='any'):
    """Sets the order of columns of a dataframe in desired order.
    Note that behavior of qualifier is same as reorder_list() function. 
    """

    keylist = reorder_list(list(df.columns), orderlist, qualifier) 
    return df.reindex_axis(keylist, axis=1)


# --------------------------------------------------------------- #
def filter_column(df, qr, col=None):
    """Filters out indices from an a DataFrame using the same 
    symantics as DataFrame indexing, and returns a new DataFrame
        df: DataFrame input (unmodified)
        qr: Simple query string that supports following forms:
              int, float, range, relational, unary & binary logical ops 
        col: if None, applies the query on the index, else matching col
    Eg, qr could be: 2, 2:3, <1, !=2, 10.5:10.75, <3 | >5, >7 & <8  
    They ALL MUST be strings though.
    Returns a an empty DataFrame if it fails.
    """
    opset = _ANYOP_.findall(qr)
    inner_eval = False
    col = 'index' if col==None else col

    if opset: 
        inner_eval = True
        if '&' in opset:
            qs = qr.split('&')
            qs = '(df.{} {}) & (df.{} {})'.format(col, qs[0], col, qs[1])
        elif '|' in opset:
            qs = qr.split('|')
            qs = '(df.{} {}) | (df.{} {})'.format(col, qs[0], col, qs[1])
        else:
            qs = '(df.{} {})'.format(col, qr) 
            print 'DEBUG', qs
    else:
        if col == 'index':
            qs = 'df.ix[{}]'.format(qr) 
        else:
            logger.error("Could not process evaluation of '{}. Returning empty DF'".format(qs))
            return pd.DataFrame( {} ) 

    logger.debug('Evaluating {}'.format(qs)) 

    try:
        dfout = df[eval(qs)] if inner_eval else eval(qs)
    except:
        logger.error("Could not process evaluation of '{}. Returning empty DF'".format(qs))
        dfout = pd.DataFrame( {} ) 

    return dfout # either a dataframe or a series


## ------------------------------------------------------------------- ##
def columnize(tbl, strip_header=True):   
    """Wrapper for converting a table in list of row (vector) format into list of column 
    (vector) format by slicing it. M rows of N-elem list/tuple would return N-rows of M-elem
    lists eg:  100 rows x (1x3) tuples => 3 rows of (100x1) vectors. So:
        
    [['Strike', 'Bid',  'Ask',    [np.array(73.0, 73.5, 74.0),
     [73.0 ,  2.65, 2.70,      =>  np.array(2.65, 2.47, 2.30), 
     [73.5 ,  2.47, 2.52,          np.array(2.70, 2.52, 2.36)]
     [74.0 ,  2.52, 2.36]]         
     NOTE the conversion of floats to strs in the transposed format is not kept by
     default IF the header is included ie, strip_header=False. Some massaging needed then. 
     """

    if len(tbl)==0 or (len(tbl)==1 and strip_header): 
        logger.error('Empty table entered. No action taken') 
        return []

    if strip_header: amat = np.matrix( tbl[1:] )    # MUST be a consistent 2-D table
    else: amat = np.matrix( tbl )    

    # n=inject to preserve native data types - except int->float
    row_types = [type(a) for a in tbl[1]]
    row_types = [float if a==int else a for a in row_types] 
     
    clist = amat.transpose().tolist()
    return [np.array(c, dtype=ntype) for c,ntype in zip(clist,row_types)] 



## ------------------------------------------------------------------- ##
def broadcast(alist, n, axis=1):
    """Broadcasts a list like object along columns (axis=1) or along rows (axis=0)
        alist: list like object, pandas pandas.Series, or numpy.array 
        n    :    int - number of times to replicate
        axis : 0|1   - 0: work down rows, 1: work along columns
    Works better with numpy.ndarray & pd.Series objects.
    NOTE. if list has non-homogenous types, typecasting will be performed as per numpy rules

    Eg: broadcast([1 2 3 4], 3, axis=0) ->  [[1, 2, 3, 4],[1, 2, 3, 4],[1, 2, 3, 4]]  - 3x4
        broadcast([1 2 3 4], 3, axis=1) ->  [[1, 1, 1],[2, 2, 2],[3, 3, 3],[4, 4, 4]]  - 4x3

    A list|narray|matrix type returns the same type back but a Series returns a DataFrame 
    """
 
    aa = ma = sa = alist    # just for convenience 

    if type(alist)==list:
        if axis==0:     return [alist]*n # easy
        else:           return (np.array([alist]).T*np.ones(n)).tolist()

    elif type(aa)==np.ndarray:
        if axis==0:     return np.array( np.mat(np.ones(n)).T*aa )
        else:           return np.array([aa]).T*np.ones(n) 

    elif type(ma)==np.matrix:
        if axis==0:     return np.mat(np.ones(n)).T*alist
        else:           return alist.T*np.ones(n)

    elif type(sa)==pd.Series:
        if axis==0:     return pd.DataFrame( np.mat(np.ones(n)).T*np.mat(sa) )
        else:           return pd.DataFrame( np.mat(aa).T*np.ones(n) )

    else:
        print 'Unrecognized list like object', type(alist) , 'entered' 
        return None


## =================================================================== ##
if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    df = parse2df('../testdir/test_parse2df.txt')
    print df

